=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-09-11 01:24:09 +0000
+++ bzrlib/branch.py	2007-09-18 20:04:21 +0000
@@ -505,7 +505,8 @@
             raise errors.NoSuchRevision(self, revno)
         return history[revno - 1]
 
-    def pull(self, source, overwrite=False, stop_revision=None):
+    def pull(self, source, overwrite=False, stop_revision=None,
+             possible_transports=None):
         """Mirror source into this branch.
 
         This branch is considered to be 'local', having low latency.
@@ -1462,7 +1463,7 @@
 
     @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
-             _hook_master=None, run_hooks=True):
+             _hook_master=None, run_hooks=True, possible_transports=None):
         """See Branch.pull.
 
         :param _hook_master: Private parameter - set the branch to 
@@ -1666,7 +1667,7 @@
         
     @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
-             run_hooks=True):
+             run_hooks=True, possible_transports=None):
         """Pull from source into self, updating my master if any.
         
         :param run_hooks: Private parameter - if false, this branch
@@ -1677,7 +1678,7 @@
         master_branch = None
         if bound_location and source.base != bound_location:
             # not pulling from master, so we need to update master.
-            master_branch = self.get_master_branch()
+            master_branch = self.get_master_branch(possible_transports)
             master_branch.lock_write()
         try:
             if master_branch:

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-09-11 00:27:26 +0000
+++ bzrlib/builtins.py	2007-09-18 20:04:21 +0000
@@ -604,8 +604,10 @@
             tree_to = None
             branch_to = Branch.open_containing(directory)[0]
 
+        possible_transports = []
         if location is not None:
             mergeable, location_transport = _get_mergeable_helper(location)
+            possible_transports.append(location_transport)
 
         stored_loc = branch_to.get_parent()
         if location is None:
@@ -617,7 +619,8 @@
                         self.outf.encoding)
                 self.outf.write("Using saved location: %s\n" % display_url)
                 location = stored_loc
-                location_transport = transport.get_transport(location)
+                location_transport = transport.get_transport(
+                    location, possible_transports=possible_transports)
 
         if mergeable is not None:
             if revision is not None:
@@ -643,8 +646,11 @@
         if verbose:
             old_rh = branch_to.revision_history()
         if tree_to is not None:
+            change_reporter = delta._ChangeReporter(
+                unversioned_filter=tree_to.is_ignored)
             result = tree_to.pull(branch_from, overwrite, revision_id,
-                delta._ChangeReporter(unversioned_filter=tree_to.is_ignored))
+                                  change_reporter,
+                                  possible_transports=possible_transports)
         else:
             result = branch_to.pull(branch_from, overwrite, revision_id)
 
@@ -2251,7 +2257,7 @@
         return '\n'.join(properties)
 
     def run(self, message=None, file=None, verbose=False, selected_list=None,
-            unchanged=False, strict=False, local=False, fixes=None,
+            unchanged=False, strict=False, local=False, fixes=[],
             author=None, show_diff=False):
         from bzrlib.commit import (
             NullCommitReporter,

=== modified file 'bzrlib/tests/commands/__init__.py'
--- bzrlib/tests/commands/__init__.py	2007-09-10 09:13:50 +0000
+++ bzrlib/tests/commands/__init__.py	2007-09-18 20:04:21 +0000
@@ -29,6 +29,7 @@
         'bzrlib.tests.commands.test_branch',
         'bzrlib.tests.commands.test_cat',
         'bzrlib.tests.commands.test_checkout',
+        'bzrlib.tests.commands.test_commit',
         'bzrlib.tests.commands.test_init',
         'bzrlib.tests.commands.test_init_repository',
         'bzrlib.tests.commands.test_merge',

=== added file 'bzrlib/tests/commands/test_commit.py'
--- bzrlib/tests/commands/test_commit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/commands/test_commit.py	2007-09-18 20:04:21 +0000
@@ -0,0 +1,60 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from bzrlib import (
+    branch,
+    builtins,
+    errors,
+    )
+from bzrlib.tests import transport_util
+
+
+class TestCommitWithBoundBranch(
+    transport_util.TestCaseWithConnectionHookedTransport):
+
+    def setUp(self):
+        super(TestCommitWithBoundBranch, self).setUp()
+        self.master_wt = self.make_branch_and_tree('master')
+        self.local_wt = self.make_branch_and_tree('local')
+
+        master_branch = branch.Branch.open(self.get_url('master'))
+        self.local_wt.branch.bind(master_branch)
+
+    def test_commit_mine_modified(self):
+
+        self.start_logging_connections()
+
+        commit = builtins.cmd_commit()
+        # commit do not provide a directory parameter, we have to change dir
+        # manually
+        os.chdir('local')
+        commit.run(message=u'empty commit', unchanged=True)
+        self.assertEquals(1, len(self.connections))
+
+    def test_commit_both_modified(self):
+        self.master_wt.commit('empty commit on master')
+        self.start_logging_connections()
+
+        commit = builtins.cmd_commit()
+        # commit do not provide a directory parameter, we have to change dir
+        # manually
+        os.chdir('local')
+        # cmd_commit translates BoundBranchOutOfDate into BzrCommandError
+        self.assertRaises(errors.BzrCommandError, commit.run,
+                          message=u'empty commit', unchanged=True)
+        self.assertEquals(1, len(self.connections))
+

=== modified file 'bzrlib/tests/commands/test_pull.py'
--- bzrlib/tests/commands/test_pull.py	2007-09-11 14:47:02 +0000
+++ bzrlib/tests/commands/test_pull.py	2007-09-18 20:04:21 +0000
@@ -15,12 +15,15 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from bzrlib.builtins import cmd_pull
-from bzrlib.tests import StringIOWrapper
-from bzrlib.tests.transport_util import TestCaseWithConnectionHookedTransport
-
-
-class TestPull(TestCaseWithConnectionHookedTransport):
+from bzrlib import (
+    branch,
+    builtins,
+    tests,
+    )
+from bzrlib.tests import transport_util
+
+
+class TestPull(transport_util.TestCaseWithConnectionHookedTransport):
 
     def test_pull(self):
         wt1 = self.make_branch_and_tree('branch1')
@@ -29,9 +32,27 @@
 
         self.start_logging_connections()
 
-        cmd = cmd_pull()
+        cmd = builtins.cmd_pull()
         # We don't care about the ouput but 'outf' should be defined
-        cmd.outf = StringIOWrapper()
+        cmd.outf = tests.StringIOWrapper()
         cmd.run(self.get_url('branch1'), directory='branch2')
         self.assertEquals(1, len(self.connections))
 
+    def test_pull_with_bound_branch(self):
+
+        master_wt = self.make_branch_and_tree('master')
+        local_wt = self.make_branch_and_tree('local')
+        master_branch = branch.Branch.open(self.get_url('master'))
+        local_wt.branch.bind(master_branch)
+
+        remote_wt = self.make_branch_and_tree('remote')
+        remote_wt.commit('empty commit')
+
+        self.start_logging_connections()
+
+        pull = builtins.cmd_pull()
+        # We don't care about the ouput but 'outf' should be defined
+        pull.outf = tests.StringIOWrapper()
+        pull.run(self.get_url('remote'), directory='local')
+        self.assertEquals(1, len(self.connections))
+

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-09-10 12:12:47 +0000
+++ bzrlib/workingtree.py	2007-09-18 20:04:21 +0000
@@ -1462,7 +1462,7 @@
 
     @needs_write_lock
     def pull(self, source, overwrite=False, stop_revision=None,
-             change_reporter=None):
+             change_reporter=None, possible_transports=None):
         top_pb = bzrlib.ui.ui_factory.nested_progress_bar()
         source.lock_read()
         try:
@@ -1470,7 +1470,8 @@
             pp.next_phase()
             old_revision_info = self.branch.last_revision_info()
             basis_tree = self.basis_tree()
-            count = self.branch.pull(source, overwrite, stop_revision)
+            count = self.branch.pull(source, overwrite, stop_revision,
+                                     possible_transports=possible_transports)
             new_revision_info = self.branch.last_revision_info()
             if new_revision_info != old_revision_info:
                 pp.next_phase()

